@@ -19,8 +19,11 @@
 
 #include "Common.hpp"
 
+#include "database/Image.hpp"
+
 namespace lms::db::tests
 {
+    using ScopedImage = ScopedEntity<db::Image>;
     using ScopedLabel = ScopedEntity<db::Label>;
     using ScopedReleaseType = ScopedEntity<db::ReleaseType>;
 
@@ -667,6 +670,58 @@ namespace lms::db::tests
         }
     }
 
+    TEST_F(DatabaseFixture, Release_releaseArtist)
+    {
+        ScopedRelease release{ session, "MyRelease" };
+        ScopedTrack track{ session };
+        ScopedArtist artist{ session, "MyArtist" };
+
+        {
+            auto transaction{ session.createReadTransaction() };
+
+            const auto releases{ Release::findIds(session, Release::FindParameters{}.setArtist(artist.getId(), { TrackArtistLinkType::ReleaseArtist })) };
+            EXPECT_EQ(releases.results.size(), 0);
+            EXPECT_EQ(Release::getCount(session, Release::FindParameters{}.setArtist(artist.getId(), { TrackArtistLinkType::ReleaseArtist })), 0);
+            EXPECT_EQ(Release::getCount(session, Release::FindParameters{}.setArtist(artist.getId())), 0);
+            EXPECT_EQ(release->getArtists(TrackArtistLinkType::ReleaseArtist).size(), 0);
+            EXPECT_EQ(release->getArtistIds(TrackArtistLinkType::ReleaseArtist).size(), 0);
+        }
+
+        {
+            auto transaction{ session.createWriteTransaction() };
+            track.get().modify()->setRelease(release.get());
+            TrackArtistLink::create(session, track.get(), artist.get(), TrackArtistLinkType::ReleaseArtist);
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+
+            auto artists{ release->getArtists(TrackArtistLinkType::ReleaseArtist) };
+            ASSERT_EQ(artists.size(), 1);
+            EXPECT_EQ(artists.front()->getId(), artist.getId());
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+
+            auto artists{ release->getArtistIds(TrackArtistLinkType::ReleaseArtist) };
+            ASSERT_EQ(artists.size(), 1);
+            EXPECT_EQ(artists.front(), artist.getId());
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+
+            EXPECT_EQ(Release::getCount(session, Release::FindParameters{}), 1);
+
+            const auto releases{ Release::findIds(session, Release::FindParameters{}.setArtist(artist.getId(), { TrackArtistLinkType::ReleaseArtist })) };
+            ASSERT_EQ(releases.results.size(), 1);
+            EXPECT_EQ(releases.results.front(), release.getId());
+            EXPECT_EQ(Release::getCount(session, Release::FindParameters{}.setArtist(artist.getId(), { TrackArtistLinkType::ReleaseArtist })), 1);
+            EXPECT_EQ(Release::getCount(session, Release::FindParameters{}.setArtist(artist.getId())), 1);
+        }
+    }
+
     TEST_F(DatabaseFixture, Release_getDiscCount)
     {
         ScopedRelease release{ session, "MyRelease" };
@@ -749,6 +804,43 @@ namespace lms::db::tests
         }
     }
 
+    TEST_F(DatabaseFixture, Label_orphan)
+    {
+        ScopedLabel label{ session, "MyLabel" };
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto labels{ Label::findOrphanIds(session) };
+            ASSERT_EQ(labels.results.size(), 1);
+            EXPECT_EQ(labels.results.front(), label.getId());
+        }
+
+        ScopedRelease release{ session, "MyRelease" };
+
+        {
+            auto transaction{ session.createWriteTransaction() };
+            release.get().modify()->addLabel(label.get());
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto labels{ Label::findOrphanIds(session) };
+            EXPECT_EQ(labels.results.size(), 0);
+        }
+
+        {
+            auto transaction{ session.createWriteTransaction() };
+            release.get().modify()->clearLabels();
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto labels{ Label::findOrphanIds(session) };
+            ASSERT_EQ(labels.results.size(), 1);
+            EXPECT_EQ(labels.results.front(), label.getId());
+        }
+    }
+
     TEST_F(DatabaseFixture, ReleaseType)
     {
         {
@@ -766,6 +858,44 @@ namespace lms::db::tests
         }
     }
 
+    TEST_F(DatabaseFixture, ReleaseType_orphan)
+    {
+        // Orphan tests
+        ScopedReleaseType releaseType{ session, "album" };
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto releaseTypes{ ReleaseType::findOrphanIds(session) };
+            ASSERT_EQ(releaseTypes.results.size(), 1);
+            EXPECT_EQ(releaseTypes.results.front(), releaseType.getId());
+        }
+
+        ScopedRelease release{ session, "MyRelease" };
+
+        {
+            auto transaction{ session.createWriteTransaction() };
+            release.get().modify()->addReleaseType(releaseType.get());
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto releaseTypes{ ReleaseType::findOrphanIds(session) };
+            EXPECT_EQ(releaseTypes.results.size(), 0);
+        }
+
+        {
+            auto transaction{ session.createWriteTransaction() };
+            release.get().modify()->clearReleaseTypes();
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto releaseTypes{ ReleaseType::findOrphanIds(session) };
+            ASSERT_EQ(releaseTypes.results.size(), 1);
+            EXPECT_EQ(releaseTypes.results.front(), releaseType.getId());
+        }
+    }
+
     TEST_F(DatabaseFixture, Release_releaseType)
     {
         ScopedRelease release{ session, "MyRelease" };
@@ -963,4 +1093,28 @@ namespace lms::db::tests
             EXPECT_EQ(release3->getTrackCount(), 0);
         }
     }
+
+    TEST_F(DatabaseFixture, Release_image)
+    {
+        ScopedRelease release{ session, "MyRelease" };
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            EXPECT_FALSE(release.get()->getImage());
+        }
+
+        ScopedImage image{ session, "/myImage" };
+
+        {
+            auto transaction{ session.createWriteTransaction() };
+            release.get().modify()->setImage(image.get());
+        }
+
+        {
+            auto transaction{ session.createReadTransaction() };
+            auto releaseImage(release.get()->getImage());
+            ASSERT_TRUE(releaseImage);
+            EXPECT_EQ(releaseImage->getId(), image.getId());
+        }
+    }
 } // namespace lms::db::tests
\ No newline at end of file
