@@ -26,6 +26,9 @@
 
 namespace lms::api::subsonic
 {
+    Response handleGetLyrics(RequestContext& context);
+    Response handleGetLyricsBySongId(RequestContext& context);
+
     void handleDownload(RequestContext& context, const Wt::Http::Request& request, Wt::Http::Response& response);
     void handleStream(RequestContext& context, const Wt::Http::Request& request, Wt::Http::Response& response);
     void handleGetCoverArt(RequestContext& context, const Wt::Http::Request& request, Wt::Http::Response& response);
